import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  arc,
  pie,
  scaleBand,
  scaleLinear,
  max,
  axisLeft,
  axisBottom,
  format,
} from 'd3';
import { selectColor } from './selectColor';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const height = 500;
const width = 960;
const padding = 20;
const centerX = width / 2;
const centerY = height / 2;
const margin = {
  top: 20,
  bottom: 70,
  left: 200,
  right: 30,
};
const xAxisLabelOffset = 50;
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;
const yValue = (d) => d.Country;
const xValue = (d) => d.Population;

const siFormat = () => format('.3s');
const xAxisTickFormat = tickValue => siFormat('.2s')(tickValue).replace('G', 'B');

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .paddingInner(0.15);

  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth]);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
        />
        <AxisLeft yScale={yScale} />
        <text
          x={innerWidth / 2}
          style={{ textAnchor: 'middle' }}
          y={innerHeight + xAxisLabelOffset}
          className="axis-label"
        >
          Population
        </text>
        <Marks
          xScale={xScale}
          yScale={yScale}
          data={data}
          selectColor={selectColor}
          xValue={xValue}
          yValue={yValue}
          tooltipFormat={xAxisTickFormat}
        />
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
